/** \file glissando.cpp
 * The main file of GLISSANDO 2
 * 
 */


/*! \mainpage
                                                                                         
            GLISSANDO - GLauber Initial State Simulation AND mOre...         \n
                        ver. 2.07, 25 August 2010                            \n
                                                                             \n 
  Authors: 
           - Wojciech Broniowski (Wojciech.Broniowski@ifj.edu.pl)              
           - Maciej Rybczynski   (Maciej.Rybczynski@pu.kielce.pl)              
           - Piotr Bozek         (Piotr.Bozek@ifj.edu.pl)                    \n  
                                                                             \n
  
  Modification of the code to ver. 2 by WB.

  For the detailed description of ver. 1 program and further references         
  to the description of the model, please, refer to our                                                                                                   
  Computer Physics Communications 180(2009)69, arXiv:0710.5731 [nucl-th]     \n
  accessibile from: http://arxiv.org.abs/0710.5731                           \n
                                                                             \n
  Implementation of nuclear correlations in ver. 2 as described in 
  Phys. Rev. C81(2010)064909                                                 \n
                                                                             \n
  Homepage: http://www.pu.kielce.pl/homepages/mryb/GLISSANDO/index.html      \n
  
  GLISSANDO is a Glauber Monte-Carlo generator for early-stages of relativistic 
  heavy-ion collisions, written in c++ and interfaced to ROOT. Several models 
  are implemented: the wounded-nucleon model, the binary collisions model, 
  the mixed model, and the model with hot-spots. The original geometric 
  distribution of sources (i.e., wounded nucleons or binary collisions) in 
  the transverse plane can be superimposed with a statistical distribution 
  simulating the dispersion in the generated transverse energy in each individual 
  collision. The program generates inter alia the fixed axes (standard) 
  and variable-axes (participant) two-dimensional profiles of the density 
  of sources in the transverse plane and their Fourier components. These profiles 
  can be used in further analyses of physical phenomena, such as the the jet 
  quenching, event-by-event hydrodynamics, or analyses of the elliptic flow 
  and its fluctuations. Characteristics of the event (multiplicities, eccentricities, 
  Fourier shape coefficients, etc.) are evaluated and stored in a ROOT file 
  for further off-line studies. A number of scripts is provided for that purpose. 
  The code can also be used for the proton-nucleus and deuteron-nucleus collisions. \n
                                                                             \n

  Version 2 of GLISSANDO offers much more functionality than version 1, 
  moreover, it is fully object-oriented, providing the user with the flexibility 
  of inspecting and, if needed, modyfying the code in a simple manner. New 
  features involve:                                                          \n
                                                                             \n
- The possibility of feeding into the simulations the nuclear distributions 
  accounting for the two-body NN correlations (read from external files, see
  Alvioli, Drescher and Strikman, [Phys. Lett. B680, 225, 2009], the distributions 
  can be found at http://www.phys.psu.edu/~malvioli/eventgenerator/ )  
- The use of the Gaussian NN wounding profile (which is more realistic than 
  the commonly-used hard-core wounding profile, see the analysis by Bialas 
  and Bzadak [Acta Phys. Polon. B38, 159,2007]) 
- The generation of the core-mantle (core-corona) distributions (see Bozek 
  [Acta Phys. Polon. B36, 3071,2005] and Werner [Phys. Rev. Lett. 98, 152301, 
  2007], see also Becattini and Manninen [Phys. Lett. B673, 19, 2009] and Bozek 
  [Phys. Rev. C79, 054901, 2009]) 
- The analysis of the triangular shape deformation parameter and profiles, 
  relevant for the triangular flow, see Alver and Roland, [Phys. Rev. C81, 054905, 
  2010] and Alver, Gombeaud, Luzum,and Ollitrault, [arXiv:1007.5469] 
- Generation of rapidity distributions in the wounded-nucleon picture according 
  to the model of Bialas and Czyz [Acta Phys.Polon.B36:905-918,2005], as implemented 
  by Bozek [arXiv:1002.4999]. This allows to obtain the fully 3-dimensional
  distribution of matter in the early Glauber phase of the collision.        \n 
                                                                          \n \n
  The reference manual for ver. 2, generated by Doxygen, is supplied at the home
  page. The full write-up of ver. 2 is under preparation.                    \n
                                                                             \n
  The code can be freely used and redistributed. However, if you decide to  
  make modifications, the authors would appreciate notification for the record.             
  Any publication or display of results obtained using GLISSANDO must        
  include a reference to our published paper.                                

*/

float ver=2.07; //!< version of the code

#include <math.h>
#include <time.h>
#include <string.h>
#include <iostream>
#include <fstream>

#include <TH1D.h>
#include <TH2D.h>
#include <TH3D.h>
#include <TFile.h>
#include <TTree.h>
#include <TRandom3.h>

#include "functions.h"
#include "distrib.h"

TRandom3 raa; //!< declare the ROOT random number generator 

using namespace std;

//! the main function of GLISSANDO
int main(
        int argc, 
//! used for passing input and output file names
/* first argument: <input.dat>
    second argument: <output.root>
    third argument: <nucl_A.dat>
    fourth argument: <nucl_B.dat> */
        char* argv[]
        ) {

/***********************
 introductory stuff
***********************/

//! (the units for all dimensionful quantities are powers of fm)

int ts; 
//! start time
ts=time_start(); 

//! print basic info 
helper(argc, argv[1]);

//! print header
header();

//! set the input file
// the input file can be passed as the first command-line argument
TString inpfile("input.dat"); 
if (argc>1) inpfile = argv[1];

//! process input parameters
readpar(inpfile);

//! set the ROOT output file
// the ROOT output file with the results can be passed as the second command-line argument
TFile *fout;
TString outfile("glissando.root"); 
if (argc>2) outfile = argv[2];
fout = new TFile(outfile,"RECREATE"); fout->cd();
cout << endl << "generates Root output file " << outfile;

//! seed the ROOT random-number generator
raa.SetSeed(ISEED); 
ISEED1 = ISEED;
ISEED = raa.GetSeed();

//! reset counters used for some basic physical quantities
reset_counters(); 

//! declare and initialize trees and histograms for storage of data
tr_his_c tr_his; // basic storage object for ROOT trees and histograms
tr_his.init();   // initialize

//! set the minimum wounding and binary-collision distances 
d=sqrt(SNN/PI/10.);      // the minimum wounding distance, 10 converts to fm^2 from mb
float d2=d*d;            // square of the minimum wounding distance
// dbin=sqrt(SBIN/PI/10.);  // the minimum binary collision distance (= the wounding distance)
dbin=d;                  // the minimum binary collision distance (note that in our model choice it is equal to the wounding distance)
float dbin2=dbin*dbin;   // square of the minimum binary collision distance 
float mbin=SBIN/SNN;     // ratio for the hot-spot model (if mbin<1, only a fraction of mbin binary collisions is included, but with the RDS enhance by the factor 1/mbin)

int kks=int(fmax(1,fmin(1000,EVENTS/10))); // for output of the progress of the run made once in a while

//! echo basic parameters to the console
echopar();

/****************************************************************************************************************/
//! #if(_files_) then initialize the nucleon distributions from external tables
/****************************************************************************************************************/

#if(_files_) // initialize tables for the nucleon distributions

FILE *nucleusA, *nucleusB;

// the input files with nuclear configurations can be passed as the third and fourth command-line arguments

TString nAfile("pb208.dat");
if (argc>3) nAfile = argv[3];
nucleusA = fopen(nAfile,"rt");

TString nBfile("pb208.dat");
if (argc>4) nBfile = argv[4]; else nBfile=argv[3];
nucleusB = fopen(nBfile,"rt");

// count number of configurations stored in files
cout << "counting the length of nuclear distribution files " << endl;

int cfc=0,bfc;
while ((bfc=fgetc(nucleusA))!=EOF) cfc+=(bfc==10)?1:0;fseek(nucleusA,0,SEEK_SET);
int snuA=cfc;
if(snuA % NUMA != 0){cout << "wrong number of lines in " << nAfile << ", exiting" << endl << endl; return 0;}
nucleusA = fopen(nAfile,"rt");

cfc=0; 
while ((bfc=fgetc(nucleusB))!=EOF) cfc+=(bfc==10)?1:0;fseek(nucleusB,0,SEEK_SET);
int snuB=cfc;
if(snuB % NUMB != 0){cout << "wrong number of lines in " << nBfile << ", exiting" << endl << endl; return 0;}
nucleusB = fopen(nBfile,"rt");

float x, y, z;
int countA=0, countB=0, charge;

float* posAx=NULL; float* posBx=NULL; float* posAy=NULL; float* posBy=NULL; float* posAz=NULL; float* posBz=NULL;
posAx=new float[snuA]; posBx=new float[snuB];
posAy=new float[snuA]; posBy=new float[snuB];
posAz=new float[snuA]; posBz=new float[snuB];

cout  << "reading " << snuA/NUMA << " configurations for nucleus A from " << nAfile << endl;
for(int i=0;i<snuA;i++){
                fscanf(nucleusA,"%f %f %f %d",&x,&y,&z,&charge);
		posAx[i]=x;
	        posAy[i]=y;
	        posAz[i]=z;};

cout  << "reading " << snuB/NUMB << " configurations for nucleus B from " << nBfile << endl << endl;
for(int i=0;i<snuB;i++){
                fscanf(nucleusB,"%f %f %f %d",&x,&y,&z,&charge);
		posBx[i]=x;
	        posBy[i]=y;
	        posBz[i]=z;};
#endif

/****************************************************************************************************************
  simulation
****************************************************************************************************************/

//! declare nuclei A nad B
nucleus nucA(NUMA), nucB(NUMB); 

//! declare the collision

#if(_rapidity_)
collision_rap collAB(NUMA,NUMB); 
#else
collision collAB(NUMA,NUMB); 
#endif

//! --- start main loop over events
kk=0; // counter of events where a collision occurs
for(int k=1;kk<EVENTS;k++){ 

evall++; // attempted A+B collision (all cases, those where an NN collision occured or not)

//! generate the distributions of nucleons in nuclei A and B
#if(_files_)
nucA.set_file(posAx, posAy, posAz, snuA, NUMA); nucB.set_file(posBx, posBy, posBz, snuB, NUMB);
#else
if(NUMA>2) nucA.set_random_A(CD); else if(NUMA==2) nucA.set_deuteron(); else nucA.set_proton();
if(NUMB>2) nucB.set_random_B(CD); else if(NUMB==2) nucB.set_deuteron(); else nucB.set_proton();
#endif

//! shift nuclei to the center-of-mass frame
nucA.shift_cmx(); nucA.shift_cmy(); nucA.shift_cmz(); 
nucB.shift_cmx(); nucB.shift_cmy(); nucB.shift_cmz();

//! #if(_profile_) generate the histograms of one-body density in nucleus A and the relative NN distance 
#if(_profile_)
for(int j=0;j<nucA.n;j++){
 tr_his.rad->Fill(sqrt(nucA.x[j]*nucA.x[j]+nucA.y[j]*nucA.y[j]+nucA.z[j]*nucA.z[j]));
    
          for(int i=0;i<j;i++){
                           tr_his.rrel->Fill(sqrt((nucA.x[j]-nucA.x[i])*(nucA.x[j]-nucA.x[i])+
                                                  (nucA.y[j]-nucA.y[i])*(nucA.y[j]-nucA.y[i])+
                                                  (nucA.z[j]-nucA.z[i])*(nucA.z[j]-nucA.z[i])));
                      if(NUMA==NUMB){              
                         tr_his.rrel_u->Fill(sqrt((nucB.x[j]-nucA.x[i])*(nucB.x[j]-nucA.x[i])+
                                                  (nucB.y[j]-nucA.y[i])*(nucB.y[j]-nucA.y[i])+
                                                  (nucB.z[j]-nucA.z[i])*(nucB.z[j]-nucA.z[i])));
                                     };
                               };
                         };
#endif

//! generate the impact parameter b with the distribution proportional to b^2 in the range (BMAX, BMIN)
b=sqrt((BMAX*BMAX-BMIN*BMIN)*los()+BMIN*BMIN);

//! shift the coordinates of the nucleons in nucleus A such that the center of mass is at the point (b*NUMB/(NUMA+NUMB),0)
nucA.shift_x(b*NUMB/(NUMA+NUMB));

//! shift the coordinates of the nucleons in nucleus B such that the center of mass is at the point (-b*NUMA/(NUMA+NUMB),0)
nucB.shift_x(-b*NUMA/(NUMA+NUMB));

//! collide the nuclei, create the sources (wounded nucleons, binary collisions) and RDS
collAB.gen_RDS(nucA,nucB,d2,dbin2,mbin);

// Include the event only when the number of wounded nucleons is at least WMIN (=2 by default)
// and, additionally, lies between W0 and W1 and the relative deposited strength (RDS) lies between RDS0 and RDS1.
// We also request that the number of sources with non-zero RDS >= WMIN (relevant
// for the Poisson superposition, where RDS can be zero)

if((collAB.nwAB >= WMIN) && (collAB.nwAB >= W0) && (collAB.nwAB <= W1) && (collAB.rpa >= RDS0) && (collAB.rpa <= RDS1) && (collAB.nzw >= WMIN)){
	
kk++; // count the event in the specified window
	
// for asymmetric collisions necessarily use SHIFT=1 in the input
if(SHIFT==1){collAB.shift_cmx_w(); collAB.shift_cmy_w();};

//! generate the rapidity distribution
#if(_rapidity_)
collAB.gen_rap(NUMRAP,MAXYRAP); 
#endif

//! #if(_weight_) generate the histograms for the NN collision profiles
#if(_weight_)
for(int j=0;j<collAB.n;j++){if(collAB.c[j]!=0){tr_his.weih->Fill(collAB.w[j],1);}
                            else {tr_his.weih_bin->Fill(collAB.w[j],1);};};
#endif

//! generate various 2-dim histograms with the distributions of sources
collAB.fill_xy(tr_his.xyhist, 1./(4*BTOT*BTOT/NBIN/NBIN)/EVENTS); // all sources
collAB.fill_xy(tr_his.xyhist_mantle, 1./(4*BTOT*BTOT/NBIN/NBIN)/EVENTS, 1, 1); // mantle sources
collAB.fill_xy(tr_his.xyhist_core, 1./(4*BTOT*BTOT/NBIN/NBIN)/EVENTS, 2, 1000); // core sources

collAB.fill_polar(tr_his.c0hist, 0, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar(tr_his.c2hist, 2, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar(tr_his.c3hist, 3, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar(tr_his.c4hist, 4, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar(tr_his.c5hist, 5, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar(tr_his.c6hist, 6, 1./(2*PI)/(BTOT/NBIN)/EVENTS);

collAB.fill_polar_s(tr_his.s3hist, 3, 1./(2*PI)/(BTOT/NBIN)/EVENTS);

//! generate Fourier moments
es=collAB.eps(2); es3=collAB.eps(3); es4=collAB.eps(4); es5=collAB.eps(5); es6=collAB.eps(6);
ess=collAB.eps_s(2); es3s=collAB.eps_s(3); es4s=collAB.eps_s(4); es5s=collAB.eps_s(5); es6s=collAB.eps_s(6); 

// rotate to the frame maximizing the m=2 moment 
float pht2=collAB.phrot(2); phirot=pht2; 
collAB.rotate(pht2);

collAB.fill_xy(tr_his.xyhistr, 1./(4*BTOT*BTOT/NBIN/NBIN)/EVENTS);

collAB.fill_polar(tr_his.c0rhist, 0, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar(tr_his.c2rhist, 2, 1./(2*PI)/(BTOT/NBIN)/EVENTS);

ep=collAB.eps(2); eps=collAB.eps_s(2);

collAB.rotate(-pht2);

// rotate to the frame maximizing the m=3 moment 
float pht3=collAB.phrot(3); phirot3=pht3;
collAB.rotate(pht3);
collAB.fill_polar(tr_his.c3rhist, 4, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
collAB.fill_polar_s(tr_his.s3rhist, 3, 1./(2*PI)/(BTOT/NBIN)/EVENTS);

ep3=collAB.eps(3); ep3s=collAB.eps_s(3); 
collAB.rotate(-pht3);

// rotate to the frame maximizing the m=4 moment 
float pht4=collAB.phrot(4); phirot4=pht4;
collAB.rotate(pht4);
collAB.fill_polar(tr_his.c4rhist, 4, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
ep4=collAB.eps(4); ep4s=collAB.eps_s(4); 
collAB.rotate(-pht4);

// rotate to the frame maximizing the m=5 moment 
float pht5=collAB.phrot(5); phirot5=pht5;
collAB.rotate(pht5);

collAB.fill_polar(tr_his.c5rhist, 5, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
ep5=collAB.eps(5); ep5s=collAB.eps_s(5); 
collAB.rotate(-pht5);

// rotate to the frame maximizing the m=4 moment 
float pht6=collAB.phrot(6); phirot6=pht6;
collAB.rotate(pht6);
collAB.fill_polar(tr_his.c6rhist, 6, 1./(2*PI)/(BTOT/NBIN)/EVENTS);
ep6=collAB.eps(6); ep6s=collAB.eps_s(6); 
collAB.rotate(-pht6);

//! get some basic properties of the event
rwA=collAB.nwA; rwB=collAB.nwB; rwAB=collAB.nwAB; rbin=collAB.nbin; rhotspot=collAB.nhotspot; rpa=collAB.rpa;
sizeav=collAB.size(); xx=collAB.cmx_w();  yy=collAB.cmy_w(); 

nwounded.add(rwAB); nbinary.add(rbin); nhot.add(rhotspot); nweight.add(rpa);
estd.add(es); epart.add(ep); estd3.add(es3); epart3.add(ep3); estd4.add(es4); epart4.add(ep4);

//! fill the data in trees and histograms
	tr_his.fill_res();
        tr_his.fill_tr();

//! if(FULL) write the full event info to the file
if(FULL){for(int j=0;j<collAB.n;j++){
			tSource.X=collAB.x[j]; 
			tSource.Y=collAB.y[j]; 
			tSource.W=collAB.w[j]; 
			tSource.KK=kk; 
			tr_his.full_event->Fill();};};

}; // if((collAB.nwAB >= WMIN)...

// write the current event count
if((kk % kks)==0){cout << "\revent: " << kk << "   ("<< int(float(kk)/float(EVENTS)*100.) << "%)              "; cout.flush();};

}; // end of loop over events 

//! --- end of main loop over events

cout << "\revent: " << kk << "     ("<< 100 << "%)              "; cout.flush();
cout << endl << endl;

/****************************************************************************************************************/
//! output of results
/****************************************************************************************************************/

// the total cross section and equivalen hard-sphere radius
sitot=PI*BMAX*BMAX*EVENTS/evall*10; // 10 converts from fm^2 to mb
sirad=sqrt(sitot/PI/10.)/2.; // equivalent hard-sphere radius in fm    

//! project out the marginal distribution in the radial variable (generate the radial Fourier profiles)
tr_his.proj();
tr_his.write_d();

//! generate and write some histograms with physical quantities
tr_his.gen();

// compute some physical info
xeps=estd.mean(); xseps=sqrt(estd.var()); xepp=epart.mean(); xsepp=sqrt(epart.var());

// write out the histograms 
        tr_his.fill();
        tr_his.write();
    #if(_profile_)
        tr_his.write_r();
    #endif

    #if(_weight_)
        tr_his.write_w();

// normalize to the wounding cross section         
        float intn=collAB.w_distr->Integral();
        collAB.w_distr->Scale(1./intn*100/6.*SNN);
        collAB.w_distr->Write();

// normalize to the binary cross section         
        intn=collAB.w_distr_bin->Integral();
        collAB.w_distr_bin->Scale(1./intn*100/6.*SBIN);
        collAB.w_distr_bin->Write();
    #endif

    #if(_rapidity_)        
        collAB.rap_distr->Write();
    #endif

//! write exit info
epilog();

//! stop time
time_stop(ts);

}

